<?php

namespace GabrielGciaFdez\Entity;

use GabrielGciaFdez\Validator\Constraint as CustomAssert;
use Symfony\Component\Validator\Constraints as Assert;
use Symfony\Component\Validator\ExecutionContextInterface;
use Symfony\Component\Validator\Validation;

class EmailData
{
    /**
     * SHA1-encoded hash for the user
     *
     * @Assert\Type(type="string")
     * @Assert\Regex("#\w{40}#")
     *
     * @var string
     */
    public $userHash;

    /**
     * Email address of the sender
     *
     * @Assert\Type(type="string")
     * @Assert\Email()
     *
     * @var string
     */
    public $sender;

    /**
     * Subject of the email
     *
     * @Assert\NotBlank(),
     * @Assert\Type(type="string")
     * @Assert\Length(max="80")
     *
     * @var string
     */
    public $subject;

    /**
     * Array of recipients of the email
     *
     * @Assert\Type(type="array")
     * @Assert\Count(min="1")
     * @Assert\All({
     *     @Assert\Collection(
     *         fields = {
     *             "name"   = @Assert\Type(type="string"),
     *             "email"  = @Assert\Email()
     *         }
     *     )
     * })
     *
     * @var array
     */
    public $recipients;

    /**
     * Datetime with format YYYY-MM-DD HH:MM:SS
     *
     * @Assert\NotBlank()
     * @Assert\DateTime()
     * @CustomAssert\DateTimeRange(min="-2 hours", max="now")
     *
     * @var string
     */
    public $date;

    /**
     * Boolean flag to know if mail has links
     *
     * @Assert\NotNull()
     * @Assert\Type(type="bool")
     *
     * @var bool
     */
    public $hasLinks;

    /**
     * Number of links in the email (optional)
     *
     * @var integer
     */
    public $numOfLinks;

    public function __construct($data)
    {
        if (null === $data) {
            throw new \InvalidArgumentException('Data cannot be null');
        }
        foreach ($data as $key => $value) {
            $this->{$key} = $value;
        }
    }

    /**
     * @Assert\Callback()
     */
    public function callbackValidateUserData(ExecutionContextInterface $context)
    {
        $userHashIsValid = false;
        $senderEmailIsValid = false;

        $userHashIsValid = !empty($this->userHash) && preg_match('#\w{40}#', $this->userHash);

        if (isset($this->sender)) {
            $emailValidator = Validation::createValidator();
            $userEmailErrors = $emailValidator->validateValue($this->sender, array(
                new Assert\NotBlank(),
                new Assert\Type(array('type' => 'string')),
                new Assert\Email()
            ));
            $senderEmailIsValid = (count($userEmailErrors) === 0);
        }

        if (!$userHashIsValid && !$senderEmailIsValid) {
            $context->addViolation(
                'It is mandatory to have a valid userHash or a valid sender',
                array(),
                null
            );
        }
    }

    /**
     * @Assert\Callback()
     */
    public function callbackValidateLinks(ExecutionContextInterface $context)
    {
        if (true === $this->hasLinks) {
            $validator = Validation::createValidator();
            $errors = $validator->validateValue($this->numOfLinks, array(
                new Assert\NotBlank(),
                new Assert\Type(array('type' => 'int')),
                new Assert\Range(array('min' => 0)),
            ));

            if (count($errors) > 0) {
                $context->addViolation(
                    'It is mandatory to have a valid numOfLinks when hasLinks flag is true',
                    array(),
                    null
                );
            }
        }
    }
}